#ifndef MYX_MATH_ALMOST_EQUAL_ULPS_INL_HPP_
#define MYX_MATH_ALMOST_EQUAL_ULPS_INL_HPP_

#pragma once

#ifndef MYXLIB_HEADER_ONLY
#include <myx/math/almost_equal_ulps_and_abs.hpp>
#endif

#include <myx/math/float_cmp_types.hpp>

#include <cmath>

namespace myx {

namespace math {

MYXLIB_INLINE bool almost_equal_ulps_and_abs( const float a, const float b,
                                              const float maxAbsDiff, const int maxUlpsDiff )
{
	// Check if the numbers are really close -- needed
	// when comparing numbers near zero.
	float absDiff = fabsf( a - b );
	if ( absDiff <= maxAbsDiff )
	{
		return( true );
	}

	FloatCmp uA( a );
	FloatCmp uB( b );

	// Different signs means they do not match.
	// NOLINTNEXTLINE(cppcoreguidelines-pro-type-union-access)
	if ( uA.negative() != uB.negative() )
	{
		return( false );
	}

	// Find the difference in ULPs.
	// NOLINTNEXTLINE(cppcoreguidelines-pro-type-union-access)
	int ulpsDiff = std::abs( uA.i - uB.i );
	if ( ulpsDiff <= maxUlpsDiff )
	{
		return( true );
	}

	return( false );
} // almost_equal_ulps_and_abs


MYXLIB_INLINE bool almost_equal_ulps_and_abs( const double a, const double b,
                                              const double maxAbsDiff, const int maxUlpsDiff )
{
	// Check if the numbers are really close -- needed
	// when comparing numbers near zero.
	double absDiff = fabs( a - b );
	if ( absDiff <= maxAbsDiff )
	{
		return( true );
	}

	DoubleCmp uA( a );
	DoubleCmp uB( b );

	// Different signs means they do not match.
	// NOLINTNEXTLINE(cppcoreguidelines-pro-type-union-access)
	if ( uA.negative() != uB.negative() )
	{
		return( false );
	}

	// Find the difference in ULPs.
	// NOLINTNEXTLINE(cppcoreguidelines-pro-type-union-access)
	auto ulpsDiff = std::abs( uA.i - uB.i );
	if ( ulpsDiff <= maxUlpsDiff )
	{
		return( true );
	}

	return( false );
} // almost_equal_ulps_and_abs

} // namespace math

} // namespace myx

#endif // ifndef MYX_MATH_ALMOST_EQUAL_ULPS_INL_HPP_
